from sqlalchemy.exc import OperationalError
import logging
import time

logger = logging.getLogger(__name__)

class BaseController:
    MAX_RETRIES = 5
    BASE_DELAY = 0.1

    @staticmethod
    def _execute_with_retry(session, operation):
        """Execute a database operation with enhanced retry logic using exponential backoff"""
        for attempt in range(BaseController.MAX_RETRIES):
            try:
                result = operation()
                return result
            except OperationalError as e:
                if "database is locked" in str(e) and attempt < BaseController.MAX_RETRIES - 1:
                    delay = BaseController.BASE_DELAY * (2 ** attempt)
                    logger.warning(f"Database is locked, attempt {attempt + 1}/{BaseController.MAX_RETRIES}. Retrying in {delay:.2f}s...")
                    time.sleep(delay)
                    session.rollback()
                    continue
                raise
            except Exception as e:
                logger.error(f"Database operation failed: {str(e)}")
                session.rollback()
                raise